#include "injection_pump_global.h"
#include "injection_pump_controller.h"

double PUMP_DIAMETER = 15;
int LEDC_LOW_FREQUENCY  = 100;
int LEDC_HIGH_FREQUENCY  = 300;
double pause_per_circle = (CIRCLE_ANGLE / STEP_ANGLE) * MICRO_STEP;

void set_pump_frequency(int low, int high)
{
    LEDC_LOW_FREQUENCY = low;
    LEDC_HIGH_FREQUENCY = high;
}

void set_pump_speed(double low_speed_ml_s, double high_speed_ml_s) {
	int low_freq = 1000*(4 * low_speed_ml_s * pause_per_circle) / (LEAD * PI * PUMP_DIAMETER * PUMP_DIAMETER);
	int high_freq = 1000 * (4 * high_speed_ml_s * pause_per_circle) / (LEAD * PI * PUMP_DIAMETER * PUMP_DIAMETER);
    set_pump_frequency(low_freq,high_freq);
}

void injection_pump_init(void)
{

    // Prepare and then apply the LEDC PWM channel configuration
    ledc_channel_config_t ledc_channel = {
        .speed_mode = LEDC_MODE,
        .channel = LEDC_CHANNEL,
        .timer_sel = LEDC_TIMER,
        .intr_type = LEDC_INTR_DISABLE,
        .gpio_num = IO_PIN_STEP,
        .duty = 0, // Set duty to 0%
        .hpoint = 0};
    ESP_ERROR_CHECK(ledc_channel_config(&ledc_channel));

      // Prepare and then apply the LEDC PWM timer configuration
    ledc_timer_config_t ledc_timer = {
        .speed_mode = LEDC_MODE,
        .timer_num = LEDC_TIMER,
        .duty_resolution = LEDC_DUTY_RES,
        .freq_hz = LEDC_LOW_FREQUENCY,
        .clk_cfg = LEDC_AUTO_CLK};
    ESP_ERROR_CHECK(ledc_timer_config(&ledc_timer));

    
    ESP_ERROR_CHECK(ledc_set_duty(LEDC_MODE, LEDC_CHANNEL, LEDC_DUTY));
    ESP_ERROR_CHECK(ledc_update_duty(LEDC_MODE, LEDC_CHANNEL));
}

void injection_pump_run(double length_mm,int direction)
{
    double half_length_mm = length_mm / 2;
    double step_up_and_down_length = 0;
    int tick_cnt = 0;
    int up_down_freq = 0;
    double mid_run_time = 0;
    for (int i = LEDC_LOW_FREQUENCY; i <= LEDC_HIGH_FREQUENCY; i += 1)
    {
        tick_cnt++;
        step_up_and_down_length += ((i / pause_per_circle) * STEP_INTERVAL / 1000)*LEAD;
        if (up_down_freq == 0 && step_up_and_down_length > half_length_mm)
        {
            up_down_freq = i;
        }
    }
    step_up_and_down_length *= 2;

    if (length_mm >= step_up_and_down_length)
    {
        up_down_freq = LEDC_HIGH_FREQUENCY;
        mid_run_time = ((length_mm - step_up_and_down_length) * 1000 * pause_per_circle / LEDC_HIGH_FREQUENCY)/LEAD;
    }


    gpio_set_level(IO_PIN_DIRECTION,direction);
    gpio_set_level(IO_PIN_ENABLE,0);
    for (int i = LEDC_LOW_FREQUENCY; i <= up_down_freq; i += 1)
    {
        ledc_set_freq(LEDC_MODE, LEDC_TIMER, i);
        vTaskDelay(STEP_INTERVAL / portTICK_PERIOD_MS);
    }

    TickType_t delayTime = mid_run_time / portTICK_PERIOD_MS;
    if (delayTime != 0)
    {
        ledc_set_freq(LEDC_MODE, LEDC_TIMER, LEDC_HIGH_FREQUENCY);
        vTaskDelay(delayTime);
    }

    for (int i = up_down_freq; i >= LEDC_LOW_FREQUENCY; i -= 1)
    {
        ledc_set_freq(LEDC_MODE, LEDC_TIMER, i);
        vTaskDelay(STEP_INTERVAL / portTICK_PERIOD_MS); 
    }
    gpio_set_level(IO_PIN_ENABLE, 1);
}
